@@ -1,6 +1,5 @@
-(* $I1: Unison file synchronizer: src/clroot.ml $ *)
-(* $I2: Last modified by tjim on Wed, 15 Jan 2003 16:46:16 -0500 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/clroot.ml *)
+(* Copyright 1999-2007 (see COPYING for details) *)
 
 (*
   This file parses the unison command-line arguments that
@@ -80,9 +79,9 @@ let getProtocolSlashSlash s =
     let matched = Str.matched_string s in
     match matched with
       "file:" | "ssh:" | "rsh:" | "socket:" ->
-        raise(Invalid_argument
+        raise(Util.Fatal
                 (Printf.sprintf
-                   "ill-formed replica %s (%s must be followed by //)"
+                   "ill-formed root specification %s (%s must be followed by //)"
                    s matched))
     | _ -> None
   else None
@@ -143,8 +142,8 @@ let parseUri s =
           if len=1 then None
           else Some(String.sub s3 1 (len-1))
         else
-          raise(Invalid_argument
-                  (Printf.sprintf "ill-formed replica %s" s)) in
+          raise(Util.Fatal
+                  (Printf.sprintf "ill-formed root specification %s" s)) in
       (protocol,userOpt,hostOpt,portOpt,pathOpt)
 
 (* These should succeed *)
@@ -197,7 +196,6 @@ let sshversion = Prefs.createString "sshversion" ""
 
 (* Main external function *)
 let parseRoot string =
-  let illegal s = raise(Prefs.IllegalValue s) in
   let illegal2 s = raise(Prefs.IllegalValue
                            (Printf.sprintf
                               "%s: %s" string s)) in
