@@ -1,10 +1,10 @@
-(* $I1: Unison file synchronizer: src/os.mli $ *)
-(* $I2: Last modified by vouillon on Tue, 31 Aug 2004 11:33:38 -0400 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/os.mli *)
+(* Copyright 1999-2007 (see COPYING for details) *)
 
 val myCanonicalHostName : string
 
 val tempPath : Fspath.t -> Path.local -> Path.local
+val tempFilePrefix : string
 val includeInTempNames : string -> unit
 
 val exists : Fspath.t -> Path.local -> bool
@@ -17,15 +17,13 @@ val childrenOf : Fspath.t -> Path.local -> Name.t list
 val readLink : Fspath.t -> Path.local -> string
 val symlink : Fspath.t -> Path.local -> string -> unit
 
-val rename : Fspath.t -> Path.local -> Fspath.t -> Path.local -> unit
-val renameIfAllowed :
-  Fspath.t -> Path.local -> Fspath.t -> Path.local -> exn option
+val rename : string -> Fspath.t -> Path.local -> Fspath.t -> Path.local -> unit
 val createDir : Fspath.t -> Path.local -> Props.t -> unit
 val delete : Fspath.t -> Path.local -> unit
 
 (* We define a new type of fingerprints here so that clients of
    Os.fingerprint do not need to worry about whether files have resource
-   forks or whatever that need to be fingerprinted separately.  They can
+   forks, or whatever, that need to be fingerprinted separately.  They can
    sensibly be compared for equality using =.  Internally, a fullfingerprint
    is a pair of the main file's fingerprint and the resource fork fingerprint,
    if any. *)
