@@ -1,11 +1,12 @@
-(* $I1: Unison file synchronizer: src/stasher.ml $ *)
+(* Unison file synchronizer: src/stasher.ml *)
 (* $I2: Last modified by lescuyer *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Copyright 1999-2007 (see COPYING for details) *)
 
 (*------------------------------------------------------------------------------------*)
 (* Preferences for backing up and stashing *)
    
 let debug = Util.debug "stasher"
+let verbose = Util.debug "stasher+"
 
 let backuplocation = 
   Prefs.createString "backuplocation" "central"
@@ -134,212 +135,119 @@ let backupcurrentnot =
    "Exceptions to \\verb|backupcurrent|, like the \\verb|ignorenot| preference."
 
 let shouldBackupCurrent p =
-  let s = Path.toString p in
-  Pred.test backupcurrent s && not (Pred.test backupcurrentnot s)
+     (* BCP: removed next line [Apr 2007]: causes ALL mergeable files to be backed
+        up, which is probably not what users want -- the backupcurrent
+        switch should be used instead.
+     Globals.shouldMerge p || *)
+  (let s = Path.toString p in
+      Pred.test backupcurrent s && not (Pred.test backupcurrentnot s))
 
 (*------------------------------------------------------------------------------------*)
 
 (* NB: We use Str.regexp here because we need group matching to retrieve
    and increment version numbers from backup file names. We only use
    it here, though: to check if a path should be backed up or ignored, we
-   use Rx instead. *)
+   use Rx instead.  (This is important because the Str regexp functions are
+   terribly slow.) *)
 
-(* regular expressions for backups, based on current preferences *)
-let version_rx = "\\([0-9]+\\)"
-
-let dir_rx = ref None
-let prefix_rx = ref ""
-let suffix_rx = ref ""
-    
 (* A tuple of string option * string * string, describing a regular
    expression that matches the filenames of unison backups according
    to the current preferences. The first regexp is an option to match
-   local directory, if ever, in which backups are stored ; the second
-   one matches the prefix, the third the suffix.                     *)
+   the local directory, if any, in which backups are stored; the second
+   one matches the prefix, the third the suffix.
+
+   Note that we always use forward slashes here (rather than using backslashes
+   when running on windows) because we are constructing rx's that are going to
+   be matched against Path.t's.  (Strictly speaking, we ought to ask the Path
+   module what the path separator character is, rather than assuming it is slash,
+   but this is never going to change.)
+ *)
 let backup_rx () =
-  let sp = Prefs.read backupprefix in
+  let version_rx = "\\([0-9]+\\)" in
+  let prefix = Prefs.read backupprefix in
   let suffix = Str.quote (Prefs.read backupsuffix) in
   let (udir, uprefix) =
-    ((match Filename.dirname sp with "." -> "" | s -> (Fileutil.bs2fs s)^"/"), 
-     Filename.basename sp) in
+    ((match Filename.dirname prefix with
+      | "." -> ""
+      | s   -> (Fileutil.backslashes2forwardslashes s)^"/"), 
+     Filename.basename prefix) in
   let (dir, prefix) = 
     ((match udir with "" -> None | _ -> Some(Str.quote udir)), Str.quote uprefix) in
-  
-  if Str.string_match (Str.regexp ".*\\\\\\$VERSION.*") (prefix^suffix) 0 then begin
+  if Str.string_match (Str.regexp ".*\\\\\\$VERSION.*") (prefix^suffix) 0 then 
     (dir,
      Str.global_replace (Str.regexp "\\\\\\$VERSION") version_rx prefix,
      Str.global_replace (Str.regexp "\\\\\\$VERSION") version_rx suffix)
-  end else
-    (dir, prefix, version_rx^suffix)
+  else
+    raise (Util.Fatal "Either backupprefix or backupsuffix must contain '$VERSION'")
    
-(* This function updates the regular expressions for backups' filenames *)   
-let updateRE () = 
-  let (a,b,c) = backup_rx () in 
-  debug ( fun () -> 
-      Util.msg "Regular Expressions for backups updated:\n ";
-      Util.msg "dir_rx: %s\n prefix_rx: %s\n suffix_rx: %s\n"
-        (match a with None -> "MISSING" | Some s -> s) b c);
-  (dir_rx := a; prefix_rx := b; suffix_rx := c)
-    
 (* We ignore files whose name ends in .unison.bak, since people may still have these lying around
    from using previous versions of Unison. *)
 let oldBackupPrefPathspec = "Name *.unison.bak"
 
 (* This function creates Rx regexps based on the preferences to ignore
-   backups of old and current versions.*)
+   backups of old and current versions.  *)
 let addBackupFilesToIgnorePref () =
+  let (dir_rx, prefix_rx, suffix_rx) = backup_rx() in
   let regexp_to_rx s =
    Str.global_replace (Str.regexp "\\\\(") ""
      (Str.global_replace (Str.regexp "\\\\)") "" s) in
   let (full, dir) =
     let d = 
-      match !dir_rx with 
+      match dir_rx with 
 	None -> "/" 
       | Some s -> regexp_to_rx s in
-    let p = regexp_to_rx !prefix_rx in
-    let s = regexp_to_rx !suffix_rx in
-    (".*"^p^".*"^s, ".*"^(String.sub d 0 (String.length d - 1))) in
+    let p = regexp_to_rx prefix_rx in
+    let s = regexp_to_rx suffix_rx in
+    debug (fun() -> Util.msg "d = %s\n" d);
+    ("(.*/)?"^p^".*"^s, "(.*/)?"^(String.sub d 0 (String.length d - 1))) in
   let theRegExp = 
-    match !dir_rx with 
+    match dir_rx with 
       None   -> "Regex " ^ full 
     | Some _ -> "Regex " ^ dir in
   debug (fun () -> 
      Util.msg "New pattern being added to ignore preferences: %s\n" theRegExp);
-  let oldRE = Pred.extern Globals.ignore in
-  let newRE = theRegExp::oldBackupPrefPathspec::oldRE in
-  Pred.intern Globals.ignore newRE
-
-(*------------------------------------------------------------------------------------*)
+  Globals.addRegexpToIgnore oldBackupPrefPathspec;
+  Globals.addRegexpToIgnore theRegExp
 
 (* We use references for functions that compute the prefixes and suffixes
-   in order to avoid using functions from the Str module each time.       *)
-let prefix_string = ref (fun i -> "")
-let suffix_string = ref (fun i -> "")
+   in order to avoid using functions from the Str module each time we need them. *)
+let make_prefix = ref (fun i -> assert false)
+let make_suffix = ref (fun i -> assert false)
     
 (* This function updates the function used to create prefixes and suffixes
    for naming backup files, according to the preferences. *)
-let updatePrefixAndSuffix () =
-  let sp = Prefs.read backupprefix in
-  let suffix = Prefs.read backupsuffix in
-  
-  let version i = Printf.sprintf "%03d" i in
-  
-  let version_appears_once s mandatory=
-    let regexp = Str.regexp "\\$VERSION" in
-    match Str.full_split regexp s with
+let updateBackupNamingFunctions () =
+  let makeFun s =
+    match Str.full_split (Str.regexp "\\$VERSION") s with
       [] -> (fun _ -> "")
     | [Str.Text t] ->  
-	if mandatory then
-	  raise (Util.Fatal "Either backupprefix or backupsuffix must contain $VERSION")
-	else
-	  (fun _ -> t)
+	(fun _ -> t)
     | [Str.Delim _; Str.Text t] -> 
-	(fun i -> Printf.sprintf "%s%s" (version i) t)
+	(fun i -> Printf.sprintf "%d%s" i t)
     | [Str.Text t; Str.Delim _] ->
-	(fun i -> Printf.sprintf "%s%s" t (version i))
+	(fun i -> Printf.sprintf "%s%d" t i)
     | [Str.Text t; Str.Delim _; Str.Text t'] ->
-	(fun i -> Printf.sprintf "%s%s%s" t (version i) t')
-    | _ -> 
-	raise (Util.Fatal ("The tag $VERSION should only appear "
-			   ^"once in the backups(prefix|suffix) preferences."))
-  in
+	(fun i -> Printf.sprintf "%s%d%s" t i t')
+    | _ -> raise (Util.Fatal (
+        "The tag $VERSION should only appear "
+       ^"once in the backupprefix and backupsuffix preferences.")) in
   
-  let (dir, prefix) =
-    ((match Filename.dirname sp with "." -> "" | s -> s), Filename.basename sp) in
-  
-  let _ = version_appears_once (prefix^suffix) true in
-  prefix_string := version_appears_once prefix false;
-  suffix_string := version_appears_once suffix false;
-  debug (fun () -> 
-           Util.msg "Prefix and Suffix for backup filenames have been updated.\n")
+  make_prefix := makeFun (Prefs.read backupprefix);
+  make_suffix := makeFun (Prefs.read backupsuffix);
+  debug (fun () -> Util.msg
+    "Prefix and suffix regexps for backup filenames have been updated\n")
 	  
-(* Generates a file name for a backup file.  If backup file already exists,
-   the old file will be renamed with the count incremented.  The newest
-   backup file is always the one with version number 1, larger numbers mean
-   older files. *)
-let backupPath fspath path =
-  let fspath = 
-    match Prefs.read backuplocation with
-      "central" -> backupDirectory ()
-    | "local" -> fspath
-    |  _ -> raise (Util.Fatal ("backuplocation preference should be set to 'central'"
-			       ^ "or 'local'.")) in
-  let rec f i =
-    let (prefix, suffix) = (!prefix_string i, !suffix_string i) in
-    let tempPath = 
-      Path.addSuffixToFinalName 
-	(Path.addPrefixToFinalName path prefix) suffix in
-    if Os.exists fspath tempPath && i < Prefs.read maxbackups then
-      Os.rename fspath tempPath fspath (f (i + 1));
-    tempPath in
-  (fspath, f 1)
-    
-(* backdir may be in subdirectories of fspath, so we recursively 
-   create them in order to avoid error in system calls to Os. *)
-let rec mkdirectories fspath backdir =
-  match Path.deconstructRev backdir with
-    None -> ()
-  | Some (_, parent) ->
-      mkdirectories fspath parent;
-      let props = (Fileinfo.get false fspath Path.empty).Fileinfo.desc in
-      if not(Os.exists fspath backdir) then Os.createDir fspath backdir props
+(*------------------------------------------------------------------------------------*)
 
-(*------------------------------------------------------------------------------------*)
-	  
-(* Removes file at fspath/path and backs it up before if required.
-   To check if the file has to be backed up and to create the
-   name of the backup file, fakeFspath/fakePath is used instead.
-   This allows us to deal directly with temporary files. *)
-let removeAndBackupAsAppropriate fspath path fakeFspath fakePath =
-  debug ( fun () -> Util.msg
-      "removeAndBackupAsAppropriate: (%s,%s), for real path (%s,%s)\n"
-      (Fspath.toString fspath)
-      (Path.toString path)
-      (Fspath.toString fakeFspath)
-      (Path.toString fakePath));
-  Util.convertUnixErrorsToTransient "removeAndBackupAsAppropriate" (fun () ->
-    if not (Os.exists fspath path) then 
-      debug (fun () -> Util.msg "File %s in %s does not exist, no need to back up\n"  
-	  (Path.toString path) (Fspath.toString fspath))
-    else
-      if shouldBackup fakePath then begin
-	let (backRoot, backPath) = backupPath fakeFspath fakePath in
-	(match Path.deconstructRev backPath with
-	  None -> ()
-	| Some (_, dir) when dir = Path.empty -> ()
-	| Some (_, backdir) -> mkdirectories backRoot backdir);
-	debug (fun () -> Util.msg "Backing up [%s] in [%s] to [%s] in [%s]\n" 
-	    (Path.toString fakePath)
-	    (Fspath.toString fakeFspath)
-	    (Path.toString backPath)
-	    (Fspath.toString backRoot));
-	try Os.rename fspath path backRoot backPath
-	with
-	  _ -> 
-	    ((let info = Fileinfo.get true fspath path in
-	    match info.Fileinfo.typ with
-	      `SYMLINK ->
-		Os.symlink 
-		  backRoot backPath 
-		  (Os.readLink fspath path)
-	    | _ ->
-		Copy.localFile
-		  fspath path 
-		  backRoot backPath backPath 
-		  `Copy 
-		  info.Fileinfo.desc
-		  (Osx.ressLength info.Fileinfo.osX.Osx.ressInfo)
-		  None);
-	     Os.delete fspath path)
-      end else begin
-	debug ( fun () -> Util.msg
-	    "File %s in %s will not be backed up.\n" 
-	    (Path.toString fakePath) 
-	    (Fspath.toString fakeFspath));
-	Os.delete fspath path
-      end)
-	  
-(*------------------------------------------------------------------------------------*)
+let makeBackupName path i =
+  (* if backups are kept centrally, the current version has exactly
+     the same name as the original, for convenience. *)
+  if i=0 && Prefs.read backuplocation = "central" then
+    path
+  else
+    Path.addSuffixToFinalName 
+      (Path.addPrefixToFinalName path (!make_prefix i))
+      (!make_suffix i)
 
 let stashDirectory fspath =
   match Prefs.read backuplocation with
@@ -348,124 +256,180 @@ let stashDirectory fspath =
   |  _ -> raise (Util.Fatal ("backuplocation preference should be set"
 			     ^"to central or local."))
 	
-let findStash path i =
-  (* if backups are kept centrally, the current version has exactly
-     th same name as the original, for convenience. *)
-  if i=0 && Prefs.read backuplocation = "central" then
-    path
-  else
-    Path.addSuffixToFinalName 
-      (Path.addPrefixToFinalName path (!prefix_string i))
-      (!suffix_string i)
-      
-(* Fingerprint.file follows symlinks, and we do not want that here *)
-let digest_safe st fspath path =
-  match st with
-    `ABSENT | `DIRECTORY -> 
-      raise (Util.Fatal (Printf.sprintf 
-			   ("Trying to digest a directory or a file that doesn't exist :\n%s/%s")
-			   (Fspath.toString fspath)
-			   (Path.toString path)))
-  | `SYMLINK -> Fingerprint.string (Os.readLink fspath path)
-  | `FILE -> Fingerprint.file fspath path
-	
-let stashPath st fspath path =
-  let tempfspath = stashDirectory fspath in
-  let tempPath = findStash path 0 in
+let showContent typ fspath path =
+  match typ with
+  | `FILE -> Fingerprint.toString (Fingerprint.file fspath path)
+  | `SYMLINK -> Os.readLink fspath path
+  | `DIRECTORY -> "DIR"
+  | `ABSENT -> "ABSENT" 
 
-  let _ =
-    match Path.deconstructRev tempPath with
+(* Generates a file name for a backup file.  If backup file already exists,
+   the old file will be renamed with the count incremented.  The newest
+   backup file is always the one with version number 1, larger numbers mean
+   older files. *)
+(* BCP: Note that the way we keep bumping up the backup numbers on all existing
+   backup files could make backups very expensive if someone sets maxbackups to a
+   sufficiently large number! 
+*)
+let backupPath fspath path =
+  let sFspath = stashDirectory fspath in
+
+  let rec f i =
+    let tempPath = makeBackupName path i in
+    if Os.exists sFspath tempPath then
+      if i < Prefs.read maxbackups then
+        Os.rename "backupPath" sFspath tempPath sFspath (f (i + 1))
+      else if i >= Prefs.read maxbackups then
+        Os.delete sFspath tempPath;
+    tempPath in
+
+  let rec mkdirectories backdir =
+    verbose (fun () -> Util.msg
+      "mkdirectories %s %s\n" (Fspath.toString sFspath) (Path.toString backdir));
+    if not (Os.exists sFspath Path.empty) then
+      Os.createDir sFspath Path.empty Props.dirDefault;
+    match Path.deconstructRev backdir with
       None -> ()
-    | Some (_, dir) when dir = Path.empty ->  ()
-    | Some (_, backdir) -> mkdirectories tempfspath backdir in
+    | Some (_, parent) ->
+        mkdirectories parent;
+        let props = (Fileinfo.get false sFspath Path.empty).Fileinfo.desc in
+        if not (Os.exists sFspath backdir) then Os.createDir sFspath backdir props in
 
-  let tempSt = (Fileinfo.get true tempfspath tempPath).Fileinfo.typ in
-  if tempSt <> `ABSENT then begin
-    if Os.exists fspath path &&
-      ((tempSt <> st) ||
-      digest_safe st fspath path <> digest_safe st tempfspath tempPath) then begin
-	if shouldBackup path then 
-        (* this is safe because this is done *after* backup *)
-	  Os.delete tempfspath tempPath 
-	else begin 
-        (* we still a keep a second backup just in case something go bad *)
-	  Trace.debug "verbose" 
-	    (fun () -> Util.msg "Creating a safety backup for (%s, %s)\n" 
-		(Fspath.toString tempfspath) (Path.toString path));
-	  let olBackup = findStash path 1 in
-	  if Os.exists tempfspath olBackup then Os.delete tempfspath olBackup;
-	  Os.rename tempfspath tempPath tempfspath olBackup
-	end;
-	Some (tempfspath, tempPath)
-      end else
-      None
-  end else
-    Some (tempfspath, tempPath)
-      
-let rec stashCurrentVersion go_rec fspath path =
-  debug (fun () -> 
-    Util.msg "stashCurrentVersion of %s in %s\n" 
-      (Path.toString path) (Fspath.toString fspath));
-  Util.convertUnixErrorsToTransient "stashCurrentVersion" (fun () ->
-    if shouldBackupCurrent path then
-      let stat = (Fileinfo.get true fspath path) in
+  let path0 = makeBackupName path 0 in
+  let sourceTyp = (Fileinfo.get true fspath path).Fileinfo.typ in
+  let path0Typ = (Fileinfo.get true sFspath path0).Fileinfo.typ in
+
+  if   (   sourceTyp = `FILE && path0Typ = `FILE
+       && (Fingerprint.file fspath path) = (Fingerprint.file sFspath path0))
+    || (   sourceTyp = `SYMLINK && path0Typ = `SYMLINK
+       && (Os.readLink fspath path) = (Os.readLink sFspath path0))
+  then begin
+    debug (fun()-> Util.msg
+      "[%s / %s] = [%s / %s] = %s: no need to back up\n"
+      (Fspath.toString sFspath) (Path.toString path0)
+      (Fspath.toString fspath) (Path.toString path)
+      (showContent sourceTyp fspath path));
+    None
+  end else begin
+    debug (fun()-> Util.msg
+      "stashed [%s / %s] = %s is not equal to new [%s / %s] = %s (or one is a dir): stash!\n"
+      (Fspath.toString sFspath) (Path.toString path0)
+      (showContent path0Typ sFspath path0)
+      (Fspath.toString fspath) (Path.toString path)
+      (showContent sourceTyp fspath path));
+    let sPath = f 0 in
+    (* Make sure the parent directory exists *)
+    begin match Path.deconstructRev sPath with
+     | None -> mkdirectories Path.empty
+     | Some (_, backdir) -> mkdirectories backdir
+    end;
+    Some(sFspath, sPath) 
+  end 
+
+(*------------------------------------------------------------------------------------*)
+	  
+let backup fspath path (finalDisposition : [`AndRemove | `ByCopying]) =
+  debug (fun () -> Util.msg
+      "backup: %s / %s\n"
+      (Fspath.toString fspath)
+      (Path.toString path));
+  Util.convertUnixErrorsToTransient "backup" (fun () ->
+    let disposeIfNeeded() =
+      if finalDisposition = `AndRemove then
+        Os.delete fspath path in
+    if not (Os.exists fspath path) then 
+      debug (fun () -> Util.msg
+        "File %s in %s does not exist, so no need to back up\n"  
+        (Path.toString path) (Fspath.toString fspath))
+    else if shouldBackup path then begin
+      match backupPath fspath path with
+        None -> disposeIfNeeded()
+      | Some (backRoot, backPath) ->
+          debug (fun () -> Util.msg "Backing up %s / %s to %s in %s\n" 
+              (Fspath.toString fspath) (Path.toString path)
+              (Path.toString backPath) (Fspath.toString backRoot));
+          let byCopying() = 
+            let rec copy p backp =
+              let info = Fileinfo.get true fspath p in
+              match info.Fileinfo.typ with
+              | `SYMLINK ->
+                  debug (fun () -> Util.msg "  Copying link %s / %s to %s / %s\n"
+                    (Fspath.toString fspath) (Path.toString p)
+                    (Fspath.toString backRoot) (Path.toString backp));
+                  Os.symlink backRoot backp (Os.readLink fspath p)
+              | `FILE ->
+                  debug (fun () -> Util.msg "  Copying file %s / %s to %s / %s\n"
+                    (Fspath.toString fspath) (Path.toString p)
+                    (Fspath.toString backRoot) (Path.toString backp));
+                  Copy.localFile  fspath p  backRoot backp backp 
+                    `Copy  info.Fileinfo.desc
+                    (Osx.ressLength info.Fileinfo.osX.Osx.ressInfo)  None
+              | `DIRECTORY ->
+                  debug (fun () -> Util.msg "  Copying directory %s / %s to %s / %s\n"
+                    (Fspath.toString fspath) (Path.toString p)
+                    (Fspath.toString backRoot) (Path.toString backp));
+                  Os.createDir backRoot backp info.Fileinfo.desc;
+                  let ch = Os.childrenOf fspath p in
+                  Safelist.iter (fun n -> copy (Path.child p n) (Path.child backp n)) ch
+              | `ABSENT -> assert false in
+            copy path backPath;
+            debug (fun () -> Util.msg "  Finished copying; deleting %s / %s\n"
+              (Fspath.toString fspath) (Path.toString path));
+            disposeIfNeeded() in
+          try 
+            if finalDisposition = `AndRemove then
+              Os.rename "backup" fspath path backRoot backPath
+            else
+              byCopying()
+          with _ -> 
+            debug (fun () -> Util.msg "Rename failed -- copying instead\n");
+            byCopying()
+      end else begin
+	debug (fun () -> Util.msg "Path %s / %s does not need to be backed up\n"
+	    (Fspath.toString fspath)
+	    (Path.toString path));
+        disposeIfNeeded()
+      end)
+	  
+(*------------------------------------------------------------------------------------*)
+
+let rec stashCurrentVersion fspath path sourcePathOpt =
+  if shouldBackupCurrent path then 
+    Util.convertUnixErrorsToTransient "stashCurrentVersion" (fun () ->
+      let sourcePath = match sourcePathOpt with None -> path | Some p -> p in
+      debug (fun () -> Util.msg "stashCurrentVersion of %s (drawn from %s) in %s\n" 
+               (Path.toString path) (Path.toString sourcePath) (Fspath.toString fspath));
+      let stat = Fileinfo.get true fspath sourcePath in
       match stat.Fileinfo.typ with
 	`ABSENT -> ()
       |	`DIRECTORY ->
-	  if go_rec then begin
-	    debug (fun () -> Util.msg "Stashing recursively because file is a directory\n");
-	    ignore (Safelist.iter
-		      (fun n ->
-			let pathChild = Path.child path n in 
-			if not (Globals.shouldIgnore pathChild) then 
-			  stashCurrentVersion true fspath (Path.child path n))
-		      (Os.childrenOf fspath path))
-	  end else
-	    debug (fun () -> Util.msg "The file is a directory but no recursive stashing\n")
-      |	st ->
-	  match stashPath st fspath path with
-	    Some (stashDir, stashPath) ->
-	      if st = `SYMLINK then
-		Os.symlink 
-		  stashDir stashPath 
-		  (Os.readLink fspath path)
-	      else
-		Copy.localFile 
-		  fspath path 
-		  stashDir stashPath stashPath 
-		  `Copy 
-		  stat.Fileinfo.desc
-		  (Osx.ressLength stat.Fileinfo.osX.Osx.ressInfo)
-		  None;
-	      debug(fun () ->
-		Util.msg "%s has been stashed to %s/%s\n" 
-		  (Path.toString path)
-		  (Fspath.toString stashDir)
-		  (Path.toString stashPath))
-	  | None ->
-	      debug (fun () -> Util.msg "Stashing was not required, contents were equal.\n"))
+           assert (sourcePathOpt = None);
+           debug (fun () -> Util.msg "Stashing recursively because file is a directory\n");
+           ignore (Safelist.iter
+                     (fun n ->
+                       let pathChild = Path.child path n in 
+                       if not (Globals.shouldIgnore pathChild) then 
+                         stashCurrentVersion fspath (Path.child path n) None)
+                     (Os.childrenOf fspath path))
+      | `SYMLINK -> 
+          begin match backupPath fspath path with
+          | None -> ()
+          | Some (stashFspath,stashPath) ->
+	      Os.symlink stashFspath stashPath (Os.readLink fspath sourcePath)
+          end 
+      |	`FILE ->
+          begin match backupPath fspath path with
+          | None -> ()
+          | Some (stashFspath, stashPath) ->
+              Copy.localFile 
+                fspath sourcePath 
+                stashFspath stashPath stashPath 
+                `Copy 
+                stat.Fileinfo.desc
+                (Osx.ressLength stat.Fileinfo.osX.Osx.ressInfo)
+                None
+          end)
       
-(* let stashCurrentVersionOnRoot: Common.root -> Path.local -> unit Lwt.t = *)
-(*   Remote.registerRootCmd *)
-(*     "stashCurrentVersion" *)
-(*     (fun (fspath, path) -> *)
-(*       Lwt.return (stashCurrentVersionLocal fspath path)) *)
-    
-(* let stashCurrentVersion path = *)
-(*   match Globals.rootsInCanonicalOrder () with *)
-(*     [r1; r2] -> *)
-(*       debug (fun () ->  *)
-(* 	Util.msg "Roots are : \n\t%s\n\t%s" *)
-(* 	  (Common.root2string r1) *)
-(* 	  (Common.root2string r2) *)
-(* 	  ); *)
-(*       Lwt_unix.run ( *)
-(*       Globals.allRootsIter *)
-(* 	(fun r -> debug (fun () -> Util.msg "iter on %s\n" (Common.root2string r)); *)
-(* 	  stashCurrentVersionOnRoot r path) *)
-(* 	) *)
-(*   | _ -> raise (Util.Fatal "The server doesn't know about the roots !") *)
-
 (*------------------------------------------------------------------------------------*)    
     
 (* This function tries to find a backup of a recent version of the file at location
@@ -479,10 +443,10 @@ let getRecentVersion fspath path fingerprint =
   Util.convertUnixErrorsToTransient "getRecentVersion" (fun () ->
     let dir = stashDirectory fspath in
     let rec aux_find i =
-      let path = findStash path i in
+      let path = makeBackupName path i in
       if Os.exists dir path &&
 	(let dig = Os.fingerprint dir path (Fileinfo.get false dir path) in 
-	dig = fingerprint)
+ 	 dig = fingerprint)
       then begin
 	debug (fun () ->
 	  Util.msg "recent version %s found in %s\n" 
@@ -505,19 +469,11 @@ let getRecentVersion fspath path fingerprint =
 (* This function initializes the Stasher module according to the preferences
    defined in the profile. It should be called whenever a profile is reloaded. *)
 let initBackupsLocal () =
+  debug (fun () -> Util.msg "initBackupsLocal\n");
   translateOldPrefs ();
-  updateRE ();
   addBackupFilesToIgnorePref ();
-  updatePrefixAndSuffix ();
+  updateBackupNamingFunctions ()
   
-  (* If the preference for backuplocation is set to central
-     then we are likely to need to create this backup directory.
-     We deal for this here.             *)
-  if (Prefs.read backuplocation = "central") then
-    let backupDir = backupDirectory () in    
-    if not(Os.exists backupDir Path.empty) then
-      Os.createDir backupDir Path.empty Props.dirDefault
-	
 let initBackupsRoot: Common.root -> unit -> unit Lwt.t =
   Remote.registerRootCmd
     "initBackups"
@@ -526,6 +482,4 @@ let initBackupsRoot: Common.root -> unit -> unit Lwt.t =
 
 let initBackups () =
   Lwt_unix.run (
-  Globals.allRootsIter
-    (fun r -> initBackupsRoot r ())
-    )
+    Globals.allRootsIter (fun r -> initBackupsRoot r ()))
