@@ -1,6 +1,5 @@
-(* $I1: Unison file synchronizer: src/uigtk2.ml $ *)
-(* $I2: Last modified by bcpierce on Sun, 22 Aug 2004 22:29:04 -0400 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/uigtk2.ml *)
+(* Copyright 1999-2007 (see COPYING for details) *)
 
 open Common
 open Lwt
@@ -9,6 +8,8 @@ module Private = struct
 
 let debug = Trace.debug "ui"
 
+let myNameCapitalized = String.capitalize Uutil.myName
+
 (**********************************************************************
                            LOW-LEVEL STUFF
  **********************************************************************)
@@ -24,7 +25,7 @@ let tryAgainMessage =
 or with a remote directory.
 
 Please enter the first (local) directory that you want to synchronize."
-Uutil.myName
+myNameCapitalized
 
 (* ---- *)
 
@@ -56,7 +57,7 @@ specify here.  (Use \"%s -socket xxx\" on the remote machine to
 start the %s server.)  You must enter the host, port, and the directory
 on the remote machine (relative to the working directory of the
 %s server running on that machine)."
-Uutil.myName Uutil.myName Uutil.myName Uutil.myName Uutil.myName Uutil.myName Uutil.myName
+myNameCapitalized myNameCapitalized myNameCapitalized myNameCapitalized myNameCapitalized myNameCapitalized myNameCapitalized
 
 (**********************************************************************
  Font preferences
@@ -514,7 +515,11 @@ let statistics () =
   ignore (lst#append ["Data received"]);
   ignore (lst#append ["File data written"]);
   let style = lst#misc#style#copy in
-  style#set_font (Lazy.force fontMonospaceMedium);
+  (* BCP: Removed this on 6/13/2006 as a workaround for a bug reported
+     by Norman Ramsey.  Apparently, lablgtl2 uses Gdk.Font, which is
+     deprecated; its associated operations don't work in recent versions
+     of gtk2. *)
+  (* style#set_font (Lazy.force fontMonospaceMedium); *)
   for r = 0 to 2 do
     lst#set_row ~selectable:false r;
     for c = 1 to 3 do
@@ -765,6 +770,7 @@ let getSecondRoot () =
     let file = fileE#text in
     let user = userE#text in
     let host = hostE#text in
+    let port = portE#text in
     match !varLocalRemote with
       `Local ->
         Clroot.clroot2string(Clroot.ConnectLocal(Some file))
@@ -773,7 +779,7 @@ let getSecondRoot () =
         Clroot.ConnectByShell((if !varLocalRemote=`SSH then "ssh" else "rsh"),
                               host,
                               (if user="" then None else Some user),
-                              Some portE#text,
+                              (if port="" then None else Some port),
                               Some file))
     | `SOCKET ->
         Clroot.clroot2string(
@@ -819,7 +825,8 @@ let getPassword rootName msg =
 
   t#vbox#set_spacing 12;
 
-  let header = primaryText (Format.sprintf "Connecting to '%s'..." rootName) in
+  let header =
+    primaryText (Format.sprintf "Connecting to '%s'..." (protect rootName)) in
 
   let h1 = GPack.hbox ~border_width:6 ~spacing:12 ~packing:t#vbox#pack () in
   (* FIX: DIALOG_AUTHENTICATION is way better but is not available
@@ -827,7 +834,7 @@ let getPassword rootName msg =
   ignore (GMisc.image ~stock:(*`DIALOG_AUTHENTICATION*)`DIALOG_QUESTION ~icon_size:`DIALOG
             ~yalign:0. ~packing:h1#pack ());
   let v1 = GPack.vbox ~spacing:12 ~packing:h1#pack () in
-  ignore(GMisc.label ~markup:(header ^ "\n\n" ^ escapeMarkup msg)
+  ignore(GMisc.label ~markup:(header ^ "\n\n" ^ escapeMarkup (protect msg))
            ~selectable:true ~yalign:0. ~packing:v1#pack ());
 
   let passwordE = GEdit.entry ~packing:v1#pack ~visibility:false () in
@@ -904,7 +911,8 @@ let scanProfiles () =
              provideProfileKey filename k f info
            with Not_found -> ());
           (f, info))
-       (Safelist.filter (fun name -> not (Util.startswith name ".#"))
+       (Safelist.filter (fun name -> not (   Util.startswith name ".#"
+                                          || Util.startswith name Os.tempFilePrefix))
           (Files.ls (Fspath.toString Os.unisonDir)
              "*.prf")))
 
@@ -1041,7 +1049,7 @@ let getProfile () =
       let (profile, info) = lst#get_row_data i in
       result := Some profile;
       begin match info.roots with
-        [r1; r2] -> root1#set_text r1; root2#set_text r2;
+        [r1; r2] -> root1#set_text (protect r1); root2#set_text (protect r2);
                     tbl#misc#set_sensitive true
       | _        -> root1#set_text ""; root2#set_text "";
                     tbl#misc#set_sensitive false
@@ -1113,10 +1121,9 @@ let documentation sect =
 
 (* ------ *)
 
-let messageBox ~title ~parent ?(action = fun t -> t#destroy)
-    ?(modal = false) message =
+let messageBox ~title ?(action = fun t -> t#destroy) ?(modal = false) message =
   let utitle = transcode title in
-  let t = GWindow.dialog ~title:utitle ~parent ~modal ~position:`CENTER () in
+  let t = GWindow.dialog ~title:utitle ~modal ~position:`CENTER () in
   let t_dismiss = GButton.button ~stock:`CLOSE ~packing:t#action_area#add () in
   t_dismiss#grab_default ();
   ignore (t_dismiss#connect#clicked ~callback:(action t));
@@ -1151,7 +1158,7 @@ let twoBoxAdvanced ~title ~message ~longtext ~advLabel ~astock ~bstock =
             ~selectable:true ~yalign:0. ~packing:v1#add ());
   t#add_button_stock `CANCEL `NO;
   let cmd () =
-    messageBox ~title:"Details" ~parent:t ~modal:false longtext
+    messageBox ~title:"Details" ~modal:false longtext
   in
   t#add_button advLabel `HELP;
   t#add_button_stock `APPLY `YES;
@@ -1190,7 +1197,7 @@ let getMyWindow () =
           | None ->
               (* Used to be ~position:`CENTER -- maybe that was better... *)
               GWindow.window ~kind:`TOPLEVEL ~position:`CENTER
-                ~title:Uutil.myName () in
+                ~title:myNameCapitalized () in
   myWindow := Some(w);
   w#set_allow_grow true;
   w
@@ -1211,7 +1218,7 @@ let displayWaitMessage () =
     ignore (GMisc.image ~stock:`DIALOG_INFO ~icon_size:`DIALOG
               ~yalign:0. ~packing:h1#pack ());
     let m =
-      GMisc.label ~markup:(primaryText "Contacting server...")
+      GMisc.label ~markup:(primaryText (Uicommon.contactingServerMsg()))
         ~yalign:0. ~selectable:true ~packing:h1#add () in
     m#misc#set_can_focus false;
     let quit = GButton.button ~stock:`QUIT ~packing:bb#pack () in
@@ -1272,8 +1279,8 @@ let rec createToplevelWindow () =
       else if label="" then p
       else p ^ " (" ^ label ^ ")" in
     toplevelWindow#set_title
-      (if s = "" then Uutil.myName else
-       Format.sprintf "%s [%s]" Uutil.myName s);
+      (if s = "" then myNameCapitalized else
+       Format.sprintf "%s [%s]" myNameCapitalized s);
     let s = if s="" then "" else "Profile: " ^ s in
     profileLabel#set_text (transcodeFilename s)
   in
@@ -1328,13 +1335,13 @@ let rec createToplevelWindow () =
       ~headers_clickable:false () in
   let s = Uicommon.roots2string () in
   ignore (lst#append_column
-    (GTree.view_column ~title:(" " ^ String.sub s  0 12 ^ " ")
+    (GTree.view_column ~title:(" " ^ protect (String.sub s  0 12) ^ " ")
        ~renderer:(GTree.cell_renderer_text [], ["text", c_replica1]) ()));
   ignore (lst#append_column
     (GTree.view_column ~title:"  Action  "
        ~renderer:(GTree.cell_renderer_pixbuf [], ["pixbuf", c_action]) ()));
   ignore (lst#append_column
-    (GTree.view_column ~title:(" " ^ String.sub s  15 12 ^ " ")
+    (GTree.view_column ~title:(" " ^ protect (String.sub s  15 12) ^ " ")
        ~renderer:(GTree.cell_renderer_text [], ["text", c_replica2]) ()));
   ignore (lst#append_column
     (GTree.view_column ~title:"  Status  " ()));
@@ -1362,8 +1369,8 @@ let rec createToplevelWindow () =
       (fun i data ->
          mainWindow#set_column
            ~title_active:false ~auto_resize:true ~title:data i)
-      [| " " ^ String.sub s  0 12 ^ " "; "  Action  ";
-         " " ^ String.sub s 15 12 ^ " "; "  Status  "; " Path" |]
+      [| " " ^ protect (String.sub s  0 12) ^ " "; "  Action  ";
+         " " ^ protect (String.sub s 15 12) ^ " "; "  Status  "; " Path" |]
   in
   setMainWindowColumnHeaders();
 
@@ -1376,12 +1383,13 @@ let rec createToplevelWindow () =
       GBin.frame ~packing:(toplevelVBox#pack ~expand:false)
         ~shadow_type:`IN (*~hpolicy:`AUTOMATIC ~vpolicy:`NEVER*) () in
     let hb =GPack.hbox ~packing:sw#add () in
-    (GButton.button ~label:"View details..." ~packing:hb#add (),
+    (GButton.button ~label:"View details..."
+         ~show:false ~packing:(hb#pack ~expand:false) (),
      GText.view ~editable:false ~wrap_mode:`NONE ~packing:hb#add ())
 
   in
   detailsWindow#misc#modify_font (Lazy.force fontMonospaceMediumPango);
-  detailsWindow#misc#set_size_chars ~height:3 ~width:104 ();
+  detailsWindow#misc#set_size_chars ~height:3 ~width:112 ();
   detailsWindow#misc#set_can_focus false;
   let showDetCommand () = 
     let details =
@@ -1391,7 +1399,7 @@ let rec createToplevelWindow () =
 	  (match !theState.(row).whatHappened with
 	    Some (Util.Failed _, Some det) -> det
 	  |  _ -> "[No details available]") in
-    messageBox ~title:"Merge execution details" ~parent:(getMyWindow ()) details
+    messageBox ~title:"Merge execution details" details
   in
   ignore (showDetailsButton#connect#clicked ~callback:showDetCommand);
   
@@ -1400,7 +1408,7 @@ let rec createToplevelWindow () =
       None ->
         grSet grAction false;
         grSet grDiff false;
-	showDetailsButton#misc#set_sensitive false
+	showDetailsButton#misc#hide ()
     | Some row ->
         let (details, activate1, activate2) =
           match !theState.(row).whatHappened, !theState.(row).ri.replicas with
@@ -1422,14 +1430,16 @@ let rec createToplevelWindow () =
               (false, true, false) in
         grSet grAction activate1;
         grSet grDiff activate2;
-	showDetailsButton#misc#set_sensitive details
+        if details then
+          showDetailsButton#misc#show ()
+        else
+          showDetailsButton#misc#hide ()
   in
 
   let makeRowVisible row =
     if mainWindow#row_is_visible row <> `FULL then begin
       let adj = mainWindow#vadjustment in
-      let current = adj#value
-      and upper = adj#upper and lower = adj#lower in
+      let upper = adj#upper and lower = adj#lower in
       let v =
         float row /. float (mainWindow#rows + 1) *. (upper-.lower) +. lower
       in
@@ -1748,21 +1758,24 @@ lst_store#set ~row ~column:c_path path;
     grSet grGo false;
     grSet grRestart false;
 
+    mainWindow#clear();
+    detailsWindow#buffer#set_text "";
+
     progressBarPulse := true;
     sync_action := Some (fun () -> progressBar#pulse ());
-    let (r1,r2) = Globals.roots () in
-    let t = Trace.startTimer "Checking for updates" in
     let findUpdates () =
+      let t = Trace.startTimer "Checking for updates" in
       Trace.status "Looking for changes";
       let updates = Update.findUpdates () in
       Trace.showTimer t;
       updates in
     let reconcile updates =
       let t = Trace.startTimer "Reconciling" in
-      Recon.reconcileAll updates in
+      let reconRes = Recon.reconcileAll updates in
+      Trace.showTimer t;
+      reconRes in
     let (reconItemList, thereAreEqualUpdates, dangerousPaths) =
       reconcile (findUpdates ()) in
-    Trace.showTimer t;
     if reconItemList = [] then
       if thereAreEqualUpdates then
         Trace.status "Replicas have been changed only in identical ways since last sync"
@@ -1781,9 +1794,11 @@ lst_store#set ~row ~column:c_path path;
     progressBarPulse := false; sync_action := None; displayGlobalProgress 0.;
     grSet grGo (Array.length !theState > 0);
     grSet grRestart true;
-    if dangerousPaths <> [] then begin
-      Prefs.set Globals.batch false;
-      Util.warn (Uicommon.dangerousPathMsg dangerousPaths)
+    if Prefs.read Globals.confirmBigDeletes then begin
+      if dangerousPaths <> [] then begin
+        Prefs.set Globals.batch false;
+        Util.warn (Uicommon.dangerousPathMsg dangerousPaths)
+      end;
     end;
   in
 
@@ -1870,7 +1885,7 @@ lst_store#set ~row ~column:c_path path;
       grSet grRestart false;
 
       Trace.status "Propagating changes";
-      Transport.start ();
+      Transport.logStart ();
       let totalLength =
         Array.fold_left
           (fun l si -> Uutil.Filesize.add l (Common.riLength si.ri))
@@ -1892,28 +1907,19 @@ lst_store#set ~row ~column:c_path path;
                   catch (fun () ->
                            Transport.transportItem
                              theSI.ri (Uutil.File.ofLine i)
-                             (fun proceed title text -> 
-			    (*    Trace.status (Printf.sprintf "\n%s\n\n%s\n\n" title text); *)
+                             (fun title text -> 
 			       textDetailed := (Some text);
-			       if proceed then
-				 if Prefs.read Uicommon.confirmmerge then
-				   twoBoxAdvanced
-				     ~title:title
-				     ~message:("The merge exited successfully."
-					       ^ " Do you want to commit the changes to"
-					       ^ " the replicas ?")
-				     ~longtext:text 
-				     ~advLabel:"View details..."
-				     ~astock:`YES
-				     ~bstock:`NO
-				 else
-				   true
-			       else
-				 (okBox ~title:title ~typ:`INFO 
-				    ~message:("\nThe merge function was unsuccessful, "
-					      ^ "changes will not be commited to "
-					      ^ "the replicas.");
-				  false))
+                               if Prefs.read Uicommon.confirmmerge then
+				 twoBoxAdvanced
+				   ~title:title
+				   ~message:("Do you want to commit the changes to"
+					     ^ " the replicas ?")
+				   ~longtext:text
+				   ~advLabel:"View details..."
+				   ~astock:`YES
+				   ~bstock:`NO
+                               else 
+				 true)
                            >>= (fun () ->
                              return Util.Succeeded))
                          (fun e ->
@@ -1941,7 +1947,7 @@ lst_store#set ~row ~column:c_path path;
       Lwt_unix.run
         (loop 0 [] Common.isDeletion >>= (fun actions ->
           Lwt_util.join actions));
-      Transport.finish ();
+      Transport.logFinish ();
       Trace.showTimer t;
       Trace.status "Updating synchronizer state";
       let t = Trace.startTimer "Updating synchronizer state" in
@@ -2008,7 +2014,7 @@ lst_store#set ~row ~column:c_path path;
     (actionBar#insert_button ~text:detectCmdName
        ~icon:((GMisc.image ~stock:`REFRESH ())#coerce)
        ~tooltip:"Check for updates"
-       ~callback:detectCmd ());
+       ~callback: detectCmd ());
 
   (*********************************************************************
     Buttons for <--, M, -->, Skip
@@ -2070,7 +2076,7 @@ lst_store#set ~row ~column:c_path path;
       Some i ->
         getLock (fun () ->
           Uicommon.showDiffs !theState.(i).ri
-            (fun title text -> messageBox ~title ~parent:(getMyWindow ()) (transcode text))
+            (fun title text -> messageBox ~title (transcode text))
             Trace.status (Uutil.File.ofLine i);
           displayGlobalProgress 0.)
     | None ->
@@ -2130,7 +2136,7 @@ lst_store#set ~row ~column:c_path path;
 
   let descl =
     if loc1 = loc2 then "right to left" else
-    Printf.sprintf "from %s to %s" loc2 loc1 in
+    Printf.sprintf "from %s to %s" (protect loc2) (protect loc1) in
   let right =
     actionsMenu#add_image_item ~key:GdkKeysyms._less ~callback:leftAction
       ~image:((GMisc.image ~stock:`GO_BACK ~icon_size:`MENU ())#coerce)
@@ -2307,7 +2313,10 @@ lst_store#set ~row ~column:c_path path;
                     (String.concat ", " (Safelist.map
                                            (fun p -> "'"^(Path.toString p)^"'")
                                            failedpaths)));
-           Prefs.set Globals.paths failedpaths; detectCmd())
+           Prefs.set Globals.paths failedpaths;
+           Prefs.set Globals.confirmBigDeletes false;
+           detectCmd();
+           reloadProfile())
        "Recheck unsynchronized items");
 
   ignore (fileMenu#add_separator ());
