@@ -1,12 +1,11 @@
-(* $I1: Unison file synchronizer: src/update.ml $ *)
-(* $I2: Last modified by vouillon on Thu, 25 Nov 2004 16:01:48 -0500 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/update.ml *)
+(* Copyright 1999-2007 (see COPYING for details) *)
 
 open Common
 let (>>=)  = Lwt.(>>=)
 
 let debug = Trace.debug "update"
-let debugverbose = Trace.debug "verbose"
+let debugverbose = Trace.debug "update+"
 let debugalias = Trace.debug "rootalias"
 let debugignore = Trace.debug "ignore"
 
@@ -20,6 +19,8 @@ let debugignore = Trace.debug "ignore"
    representation does not change between unison versions.) *)
 (*FIX: change the approximate function in props.ml next time the
   format is modified (see file props.ml for the new function) *)
+(*FIX: also change Fileinfo.stamp to drop the info.ctime component, next time the
+  format is modified *)
 let archiveFormat = 22
 
 module NameMap = MyMap.Make (Name)
@@ -62,7 +63,7 @@ let archive2string = function
    A is A' + A' + B', and the canonical archive name for root B is B' + A' +
    B'.
 
-   Currently, we determine A' + B' in during startup and store this in the
+   Currently, we determine A' + B' during startup and store this in the
    ref cell rootsName, below.  This rootsName is passed as an argument to
    functions that need to determine a canonical archive name.  Note, since
    we have a client/server architecture, there are TWO rootsName ref cells
@@ -72,10 +73,11 @@ let archive2string = function
    the future; we have implemented it this way at first for historical
    reasons. *)
 
-
 let rootsName : string Prefs.t =
   Prefs.createString "rootsName" "" "*Canonical root names" ""
 
+let getRootsName () = Prefs.read rootsName
+
 let foundArchives = ref true
 
 (*****************************************************************************)
@@ -319,7 +321,7 @@ let removeArchiveOnRoot: Common.root -> archiveVersion -> unit Lwt.t =
   Remote.registerRootCmd "removeArchive" removeArchiveLocal
 
 (* [commitArchive (fspath, ())] commits the archive for [fspath] by changing
-   the filenames from ScartchArch-ones to a NewArch-ones *)
+   the filenames from ScratchArch-ones to a NewArch-ones *)
 let commitArchiveLocal ((fspath: Fspath.t), ())
     : unit Lwt.t =
   Lwt.return
@@ -398,7 +400,6 @@ let fileUnchanged oldInfo newInfo =
 
 let archiveUnchanged fspath newInfo =
   let (arcName, thisRoot) = archiveName fspath MainArch in
-  let arcFspath = Os.fileInUnisonDir arcName in
   try
     fileUnchanged (Hashtbl.find archiveInfoCache thisRoot) newInfo
   with Not_found ->
@@ -522,8 +523,11 @@ let loadArchives (optimistic: bool) : bool Lwt.t =
     raise (Util.Fatal(
 	"Internal error: On-disk archives are not identical.\n"
       ^ "\n"
-      ^ "If you get this message repeatedly, please:\n"
-      ^ "  a) Notify unison-help@cis.upenn.edu.\n"
+      ^ "This can happen when both machines have the same hostname.\n"
+      ^ "\n"
+      ^ "If this is not the case and you get this message repeatedly, please:\n"
+      ^ "  a) Send a bug report to unison-users@yahoogroups.com (you may need"
+      ^ "     to join the group before you will be allowed to post).\n"
       ^ "  b) Move the archive files on each machine to some other directory\n"
       ^ "     (in case they may be useful for debugging).\n"
       ^ "     The archive files on this machine are in the directory\n"
@@ -797,25 +801,29 @@ let doArchiveCrashRecovery () =
              ["Please delete archive files as appropriate and try again."]))))
   else begin
     foundArchives := false;
-    (* This can be nuked if we don't end up needing it pretty soon:
-    let expectedNames =
-      String.concat ""
-        (Safelist.map
-           (fun (name,host,_) ->
-              Printf.sprintf "    %s    on host %s\n" name host)
-           (Globals.allRootsMap (fun r -> archiveNameOnRoot r MainArch))) in *)
-    Util.warn
-    ("No archive files were found for these roots.  This can happen either\n"
-   ^ "because this is the first time you have synchronized these roots, \n"
-   ^ "or because you have upgraded Unison to a new version with a different\n"
-   ^ "archive format.  \n\n"
-   ^ "Update detection may take a while on this run if the replicas are \n"
-   ^ "large.\n\n"
-   ^ "Unison will assume that the 'last synchronized state' of both replicas\n"
-   ^ "was completely empty.  This means that any files that are different\n"
-   ^ "will be reported as conflicts, and any files that exist only on one\n"
-   ^ "replica will be judged as new and propagated to the other replica.\n"
-   ^ "If the two replicas are identical, then no changes will be reported.\n"
+    let expectedRoots =
+      String.concat "\n\t" (Safelist.map root2string (Globals.rootsList ())) in
+     Util.warn
+     ("No archive files were found for these roots, whose canonical names are:\n\t"
+     ^ expectedRoots ^ "\nThis can happen either\n"
+     ^ "because this is the first time you have synchronized these roots, \n"
+     ^ "or because you have upgraded Unison to a new version with a different\n"
+     ^ "archive format.  \n\n"
+     ^ "Update detection may take a while on this run if the replicas are \n"
+     ^ "large.\n\n"
+     ^ "Unison will assume that the 'last synchronized state' of both replicas\n"
+     ^ "was completely empty.  This means that any files that are different\n"
+     ^ "will be reported as conflicts, and any files that exist only on one\n"
+     ^ "replica will be judged as new and propagated to the other replica.\n"
+     ^ "If the two replicas are identical, then no changes will be reported.\n\n"
+     ^ "If you see this message repeatedly, it may be because one of your machines\n"
+     ^ "is getting its address from DHCP, which is causing its host name to change\n"
+     ^ "between synchronizations.  See the documentation for the UNISONLOCALHOSTNAME\n"
+     ^ "environment variable for advice on how to correct this.\n"
+     ^ "\n"
+     ^ "Donations to the Unison project are gratefully accepted: \n"
+     ^ "http://www.cis.upenn.edu/~bcpierce/unison\n"
+     ^ "\n"
      (* ^ "\nThe expected archive names were:\n" ^ expectedNames *) );
     Lwt.return ()
   end))
@@ -901,6 +909,30 @@ let isDir fspath path =
     (Fspath.stat fullFspath).Unix.LargeFile.st_kind = Unix.S_DIR
   with Unix.Unix_error _ -> false
 
+(***********************************************************************
+                             MOUNT POINTS
+************************************************************************)
+
+let mountpoints = 
+  Prefs.createStringList "mountpoint"
+    "abort if this path does not exist"
+    ("Including the preference \\texttt{-mountpoint PATH} causes Unison to "
+     ^ "double-check, at the end of update detection, that \\texttt{PATH} exists "
+     ^ "and abort if it does not.  This is useful when Unison is used to synchronize "
+     ^ "removable media.  This preference can be given more than once.  "
+     ^ "See \\sectionref{mountpoints}{Mount Points}.")
+
+let abortIfAnyMountpointsAreMissing fspath =
+  Safelist.iter
+    (fun s ->
+       let path = Path.fromString s in
+       if not (Os.exists fspath path) then
+         raise (Util.Fatal
+           (Printf.sprintf "Path %s / %s is designated as a mountpoint, but points to nothing on host %s\n"
+             (Fspath.toString fspath) (Path.toString path) Os.myCanonicalHostName)))
+    (Prefs.read mountpoints)
+
+
 (***********************************************************************
                            UPDATE DETECTION
 ************************************************************************)
@@ -912,13 +944,13 @@ let fastcheck =
   Prefs.createString "fastcheck" "default"
     "do fast update detection (`true', `false', or `default')"
     ( "When this preference is set to \\verb|true|, \
-       Unison will use file creation times as `pseudo inode numbers' \
+       Unison will use the modification time and length of a file as a
+       `pseudo inode number' \
        when scanning replicas for updates, \
        instead of reading the full contents of every file.  Under \
        Windows, this may cause Unison to miss propagating an update \
-       if the create time, modification time, and length of the \
-       file are all unchanged by the update (this is not easy to \
-       achieve, but it can be done).  However, Unison will never \
+       if the modification time and length of the \
+       file are both unchanged by the update.  However, Unison will never \
        {\\em overwrite} such an update with a change from the other \
        replica, since it always does a safe check for updates just \
        before propagating a change.  Thus, it is reasonable to use \
@@ -959,8 +991,8 @@ let t0 = ref 0.
 (* Note that we do *not* want to do any status displays from the server
    side, since this will cause the server to block until the client has
    finished its own update detection and can receive and acknowledge
-   the status display message -- effectively serializing the client and
-   server. *)
+   the status display message -- thus effectively serializing the client 
+   and server! *)
 let showStatusAddLength info =
   if not !Trace.runningasserver then begin
     let len1 = Props.length info.Fileinfo.desc in
@@ -980,7 +1012,7 @@ let showStatus path =
       fileLength := 0;
       let t = Unix.gettimeofday () in
       if t -. !t0 > 0.05 then begin
-        Trace.statusDetail (Path.toString path);
+        Trace.statusDetail ("scanning " ^ Path.toString path);
         t0 := t
       end
     end
@@ -1017,14 +1049,12 @@ let checkPropChange info archive archDesc =
              oldInfoOf archive)
   end
 
-(* HACK: we disable fastcheck for Excel files on Windows, as Excel
+(* HACK: we disable fastcheck for Excel (and MPP) files on Windows, as Excel
    sometimes modifies a file without updating the time stamp. *)
-let notExcelFile path =
+let excelFile path =
   let s = Path.toString path in
-  let l = String.length s in
-  l < 4 ||
-  not
-    (s.[l - 4] = '.' && s.[l - 3] = 'x' && s.[l - 2] = 'l' && s.[l - 1] = 's')
+     Util.endswith s ".xls"
+  || Util.endswith s ".mpp"
 
 (* Check whether a file has changed has changed, by comparing its digest and
    properties against [archDesc], [archDig], and [archStamp].
@@ -1038,22 +1068,22 @@ let checkContentsChange
    =
   debug (fun () ->
            Util.msg "checkContentsChange: ";
-           begin match archive with
-             ArchiveFile (_, _, Fileinfo.InodeStamp inode, _) ->
-               Util.msg "archive: stamp is inode (%d)" inode
-           | ArchiveFile (_, _, Fileinfo.CtimeStamp date, _) ->
-               Util.msg "archive : stamp is ctime (%f)" date
-           | _ -> Util.msg "archive: not a file\n"
-           end;
            begin
              match archStamp with
                Fileinfo.InodeStamp inode ->
-                 (Util.msg " / archStamp is inode (%d)" inode;
-                  Util.msg " / info.inode (%d)\n" info.Fileinfo.inode)
+                 (Util.msg "archStamp is inode (%d)" inode;
+                  Util.msg " / info.inode (%d)" info.Fileinfo.inode)
              | Fileinfo.CtimeStamp stamp ->
-                 (Util.msg " / archStamp is ctime (%f)" stamp;
-                  Util.msg " / info.ctime (%f)\n" info.Fileinfo.ctime)
-           end);
+                 (Util.msg "archStamp is ctime (%f)" stamp;
+                  Util.msg " / info.ctime (%f)" info.Fileinfo.ctime)
+           end;
+           Util.msg " / times: %f - %f - %b"
+             (Props.time archDesc) (Props.time info.Fileinfo.desc)
+             (Props.same_time info.Fileinfo.desc archDesc);
+           Util.msg " / lengths: %s - %s"
+             (Uutil.Filesize.toString (Props.length archDesc))
+             (Uutil.Filesize.toString  (Props.length info.Fileinfo.desc));
+           Util.msg "\n");
   let dataClearlyUnchanged =
     fastCheck
       &&
@@ -1061,11 +1091,16 @@ let checkContentsChange
       &&
     Props.length info.Fileinfo.desc = Props.length archDesc
       &&
+    not (excelFile path)
+      &&
     match archStamp with
       Fileinfo.InodeStamp inode ->
         info.Fileinfo.inode = inode
     | Fileinfo.CtimeStamp ctime ->
-        info.Fileinfo.ctime = ctime && notExcelFile path in
+        (* BCP [Apr 07]: This doesn't work -- ctimes are unreliable
+                         under windows.  :-(  
+           info.Fileinfo.ctime = ctime *)
+        true in
   let ressClearlyUnchanged =
     fastCheck
       &&
@@ -1108,10 +1143,6 @@ let checkContentsChange
 
    Note that case conflicts and illegal filenames can only occur under Unix,
    when syncing with a Windows file system. *)
-let someHostIsRunningWindows =
-  Prefs.createBool "someHostIsRunningWindows" false "*" ""
-let allHostsAreRunningWindows =
-  Prefs.createBool "allHostsAreRunningWindows" false "*" ""
 let badWindowsFilenameRx =
   (* FIX: This should catch all device names (like aux, con, ...).  I don't
      know what all the possible device names are. *)
@@ -1123,7 +1154,7 @@ let isBadWindowsFilename s =
   Rx.match_string badWindowsFilenameRx (Name.toString s)
 let badFilename s =
   (* Don't check unless we are syncing with Windows *)
-  Prefs.read someHostIsRunningWindows &&
+  Prefs.read Globals.someHostIsRunningWindows &&
   isBadWindowsFilename s
 
 let getChildren fspath path =
@@ -1236,9 +1267,10 @@ let rec buildUpdateChildren
           (nm, handleChild nm archive `Abs)
         else begin
           curChildren := rem;
-          if c = 0 then
+          if c = 0 then begin
+            if nm <> nm' then archUpdated := true;
             (nm', handleChild nm' archive st)
-          else begin
+          end else begin
             let arch = handleChild nm' NoArchive st in
             assert (arch = NoArchive);
             matchChild nm archive
@@ -1356,15 +1388,27 @@ let rec buildUpdate archive fspath fullpath here path =
        ui)
   | Some(name, path') ->
       if not (isDir fspath here) then
-        (archive,
-         if Path.isEmpty here then
-           Error (Printf.sprintf
-                    "path %s is not valid because the root of one of the replicas is not a directory"
-                    (Path.toString fullpath))
-         else
-           Error (Printf.sprintf
-                    "path %s is not valid because %s is not a directory in one of the replicas"
-                    (Path.toString fullpath) (Path.toString here)))
+        let error =
+          if Path.isEmpty here then
+            Printf.sprintf
+              "path %s is not valid because the root of one of the replicas \
+               is not a directory"
+              (Path.toString fullpath)
+          else
+            Printf.sprintf
+              "path %s is not valid because %s is not a directory in one of \
+               the replicas"
+              (Path.toString fullpath) (Path.toString here)
+        in
+        (* FIX: We have to fail here (and in other error cases below)
+           rather than report an error for this path, which would be
+           more user friendly.  Indeed, the archive is otherwise
+           modified in inconsistent way when the failure occurs only
+           on one replica (see at the end of this function).
+           A better solution should be not to put the archives in a
+           different state, but this is a lot more work. *)
+        raise (Util.Transient error)
+(*      (archive, Error error) *)
       else
       let children = getChildren fspath here in
       let (name', status) =
@@ -1379,8 +1423,13 @@ let rec buildUpdate archive fspath fullpath here path =
                    ("The path " ^ Path.toString fullpath ^
                     " is not allowed in Windows"))
       | `Dup ->
-          raise (Util.Transient ("The path " ^ Path.toString fullpath ^
-                                 " is ambiguous"))
+          raise (Util.Transient
+            ("The path " ^ Path.toString fullpath ^
+             " is ambiguous (i.e., the name of this path or one of its "
+             ^ "ancestors is the same, modulo capitalization, as another "
+             ^ "path in a case-sensitive filesystem, and you are "
+             ^ "synchronizing this filesystem with a case-insensitive "
+             ^ "filesystem.  ")) 
       | `Ok ->
           let (desc, child, otherChildren) =
             match archive with
@@ -1412,6 +1461,12 @@ let rec buildUpdate archive fspath fullpath here path =
 let findLocal fspath pathList: Common.updateItem list =
   debug (fun() -> Util.msg "findLocal %s\n" (Fspath.toString fspath));
   addHashToTempNames fspath;
+  (* Maybe we should remember the device number where the root lives at 
+     the beginning of update detection, so that we can check, below, that 
+     the device has not changed.  This check allows us to abort in case 
+     the root is on a removable device and this device gets removed during
+     update detection, causing all the files to appear to have been
+     deleted.  --BCP 2006 *)
   let (arcName,thisRoot) = archiveName fspath MainArch in
   let archive = getArchive thisRoot in
   let (archive, updates) =
@@ -1427,6 +1482,7 @@ let findLocal fspath pathList: Common.updateItem list =
       pathList (archive, [])
   in
   setArchiveLocal thisRoot archive;
+  abortIfAnyMountpointsAreMissing fspath;
   updates
 
 let findOnRoot =
@@ -1436,10 +1492,6 @@ let findOnRoot =
        Lwt.return (findLocal fspath pathList))
 
 let findUpdatesOnPaths pathList : Common.updateItem list Common.oneperpath =
-(*
-let t1' = Unix.times () in
-let t1 = Unix.gettimeofday () in
-*)
   Lwt_unix.run
     (loadArchives true >>= (fun ok ->
      begin if ok then Lwt.return () else begin
@@ -1452,16 +1504,7 @@ let t1 = Unix.gettimeofday () in
             unlockArchives ()) >>= (fun _ ->
        unlockArchives ()))
      end end >>= (fun () ->
-(*
-let t2 = Unix.gettimeofday () in
-let t2' = Unix.times () in
-Format.eprintf "Archive loading: %f / %f / %f@." (t2 -. t1) (t2'.Unix.tms_utime -. t1'.Unix.tms_utime) (t2'.Unix.tms_stime -. t1'.Unix.tms_stime);
-*)
      let t = Trace.startTimer "Collecting changes" in
-(*
-let t3' = Unix.times () in
-let t3 = Unix.gettimeofday () in
-*)
      Globals.allRootsMapWithWaitingAction (fun r ->
        debug (fun() -> Util.msg "findOnRoot %s\n" (root2string r));
        findOnRoot r pathList)
@@ -1471,11 +1514,6 @@ let t3 = Unix.gettimeofday () in
          | _ -> ()
          end)
        >>= (fun updates ->
-(*
-let t2 = Unix.gettimeofday () in
-let t2' = Unix.times () in
-Format.eprintf "Update detection: %f / %f / %f@." (t2 -. t3) (t2'.Unix.tms_utime -. t3'.Unix.tms_utime) (t2'.Unix.tms_stime -. t3'.Unix.tms_stime);
-*)
      Trace.showTimer t;
      let result = Safelist.transpose updates in
      Trace.status "";
@@ -1630,13 +1668,15 @@ let updateArchiveLocal fspath path ui id =
   let (localPath, subArch) = getPathInArchive archive Path.empty path in
   let newArch = updateArchiveRec ui (stripArchive path subArch) in
   let commit () =
-    let _ = Stasher.stashCurrentVersion true fspath localPath in
+    let _ = Stasher.stashCurrentVersion fspath localPath in
     let archive = getArchive root in
     let archive, () =
       updatePathInArchive archive fspath Path.empty path
         (fun _ _ _ -> newArch, ()) in
     setArchiveLocal root archive in
   setCommitAction root id commit;
+  debug (fun() ->
+    Util.msg "updateArchiveLocal --> %s\n" (Path.toString localPath));
   (localPath, newArch)
 
 let updateArchiveOnRoot =
@@ -1664,7 +1704,7 @@ let markEqualLocal fspath paths =
               let arch = updateArchiveRec (Updates (uc, New)) archive in
               arch, (arch, localPath))
        in
-       Stasher.stashCurrentVersion false fspath localPath;
+       Stasher.stashCurrentVersion fspath localPath None;
        archive := arch);
   setArchiveLocal root !archive
 
@@ -1728,7 +1768,8 @@ let replaceArchiveLocal fspath pathTo location arch id paranoid =
   in
   let newArch = replaceArchiveRec workingDir tempPathTo arch paranoid in
   let commit () =
-    let _ = Stasher.stashCurrentVersion true fspath localPath in
+    debug (fun() -> Util.msg "replaceArchiveLocal: committing\n");
+    let _ = Stasher.stashCurrentVersion fspath localPath (Some tempPathTo) in
     let archive = getArchive root in
     let archive, () =
       updatePathInArchive archive fspath Path.empty pathTo
@@ -1811,7 +1852,7 @@ let updateProps root path propOpt ui id =
    updatePropsOnRoot root (path, propOpt, ui, id)
 
 (*************************************************************************)
-(*                       Make sure no change has happened                *)
+(*                  Make sure no change has happened                     *)
 (*************************************************************************)
 
 let checkNoUpdatesLocal fspath pathInArchive ui =
